%
% creates a problem structure that is used by the LKF, Linearized KF, EKF
% and IEKF to estimate a state

pr 'creating problem structures';

% initialize KF initial state estimate and uncertainty

problem.model.state_est.value = [10; 10; 0];   
problem.model.state_est.uncert = [1,0,0; 0,1,0; 0,0,0.5]; % little bit of location uncert


% =======================================================================
% PROCESS MODEL
%
% name of the process

problem.model.process.name = 'differential driving robot';

% KF process noise
problem.model.process.noise.value = [0; 0; 0]; % zero mean noise
problem.model.process.noise.uncert = [0.5, 0, 0; 0,0.5, 0; 0,0,0.5];   % with some independent noise


% the process function evolves the states over time, 
% f(x_{k-1},v_{k-1}) = x_{k-1} +...+ v_{k-1} --> f

problem.model.process.function = @diff_drive_process_function;

% what parameters does the process funtion take?
% f(x_{k-1},v_{k-1}) --> x_{k-1} and v_{k-1}

% per parameter the following questions need to be answered:

% what function returns value and uncertainty of this parameter?
% function gets as parameter the problem model

% which function creates the transition matrix or Jacobian for this para?
% the function gets as parameters the same parameters as process function

% parameter 1
problem.model.process.para(1).name = 'last state estimate';
problem.model.process.para(1).get_value = @get_last_state_estimate;
problem.model.process.para(1).trans_matrix_func = @diff_drive_process_function_Jacobian_x;

% parameter 2
problem.model.process.para(2).name = 'last control input';
problem.model.process.para(2).get_value = @get_last_control_input;
problem.model.process.para(2).trans_matrix_func = @diff_drive_process_function_Jacobian_u;

% parameter 3
problem.model.process.para(3).name = 'process noise';
problem.model.process.para(3).get_value = @get_KF_process_noise;
problem.model.process.para(3).trans_matrix_func = @diff_drive_process_function_Jacobian_v;



% =======================================================================
% MEASUREMENT MODEL
%
% name of the measurement model

problem.model.meas.name = 'full state sensor for differential drive';

% KF measurement noise for this model
problem.model.meas.noise.value = [0; 0; 0];
%problem.model.meas.noise.uncert = [.90^2, 0, 0; 0,0.0000001,0; 0,0,0.05];
%problem.model.meas.noise.uncert = [0.20^2, 0,0; 0,0.20^2,0; 0,0,0.0005^2];
problem.model.meas.noise.uncert = [0.90^2, 0,0; 0,0.90^2,0; 0,0,0.05^2];


% the measurement function predicts measurements, h(x_k) = \hat{z}_k
problem.model.meas.function = @diff_drive_meas_function;

% what parameters does the measurement funtion take? per parameter, answer:
% what function returns value and uncertainty of this parameter?
% which function creates the sensitivity matrix or Jacobian for this para?
% the function gets as parameters the same parameters as meas function

% parameter 1
problem.model.meas.para(1).name = 'last state estimate'; % first para has to be state!
problem.model.meas.para(1).get_value = @get_last_state_estimate;
problem.model.meas.para(1).trans_matrix_func = @diff_drive_meas_function_Jacobian_x;

% parameter 2
problem.model.meas.para(2).name = 'measurement noise';
problem.model.meas.para(2).get_value = @get_KF_measurement_noise;
problem.model.meas.para(2).trans_matrix_func = @diff_drive_meas_function_Jacobian_w;



% =======================================================================
% ENVIRONMENT model
%
% name of the environment

problem.env.name = 'environment for one differential drive robot';

% PROCESS settings
%
%
% what is the amount of true process noise?
problem.env.process.noise.value = [0, 0, 0]; % zero-mean
problem.env.process.noise.uncert = [2^2, 0, 0; 0, 2^2, 0; 0,0,(0.5*((2*pi)/360))^2]; % covariance

% what function brings the process one step further?
problem.env.process.function = @diff_drive_process_step; 

% and what parameters does this function take, and 
% which function returns the value of these parameters?

% parameter 1
problem.env.process.para(1).name = 'true state';
problem.env.process.para(1).get_value = @get_last_true_state;

% parameter 2
problem.env.process.para(2).name = 'control input';
problem.env.process.para(2).get_value = @get_last_control_input;

% parameter    3
problem.env.process.para(3).name = 'true process noise';
problem.env.process.para(3).get_value = @get_last_true_process_noise;

% what is the initial true state of the proces?
problem.env.process.result.value = [10; 10; 0]; % latest step result
problem.env.process.result.uncert = [0.20^2, 0,0; 0,0.20^2,0; 0,0,0.0005^2];

% MEASUREMENT settings
%
% what is the amount of true measurement noise?
problem.env.meas.noise.value = [0;0;0];
problem.env.meas.noise.uncert = [2^2,0,0; 0,2^2,0 ; 0,0,(10*((2*pi)/360))^2];

% what function makes a real measurement?
problem.env.meas.function = @diff_drive_get_real_measurement; 

% what parameters does this function take and 
% which function gives values for them?

% parameter 1
problem.env.meas.para(1).name = 'true state';
problem.env.meas.para(1).get_value = @get_last_true_state;

% parameter 2
problem.env.meas.para(2).name = 'measurement noise';
problem.env.meas.para(2).get_value = @get_last_true_measurement_noise;

% what is the initial measurement?
problem.env.meas.result.value = [NaN;NaN;NaN]; 
problem.env.meas.result.uncert = [0.20^2, 0,0; 0,0.20^2,0; 0,0,0.0005^2];


% MISC
%
% CONTROL settings
% control makes a decision on the next action to execute
problem.env.control.noise.value = [0;0];        % no control noise
problem.env.control.noise.uncert = [0,0;0,0];

problem.env.control.function = @diff_drive_make_square_control; 
problem.env.control.para(1).name = 'true state';
problem.env.control.para(1).get_value = @get_last_true_state;

problem.env.control.para(2).name = 'true control noise';
problem.env.control.para(2).get_value = @get_last_true_control_noise;

problem.env.control.result.value = [0;0]; % latest make_control result
problem.env.control.result.uncert = [0,0;0,0]; % control noise


